# jQuery Attach/Detach

[![NPM downloads](https://img.shields.io/npm/dm/jquery-attach.svg)](https://npmjs.org/package/jquery-attach "View this project on NPM")

**jQuery-attach** is a simple jQuery plugin to ensure that elements don't get bound to an event multiple times.

## Installation

`npm install jquery-attach --save-dev`

## Basic Usage

### attachBehavior

Initiates the one-time binding

```
$('.clicker').attachBehavior(trigger, evt => {
  evt.preventDefault();
  // do your thing
});
```

Arguments:

- `trigger`: the event to be handled (e.g. `'click'`)
- `evt`: callback's event object


### detachBehavior

Removes the binding

```
$('.clicker').detachBehavior('click');
```


### getBehavior

Check what behavior's attach to an element

```
$('.clicker').attachBehavior(trigger, function(evt) {
  evt.preventDefault();
});

let clickerBehavior = $('.clicker').getBehavior();

console.log( clickerBehavior.trigger );
// Outputs: 'click'

console.log( clickerBehavior.callback );
// Outputs: 'function(evt) { evt.preventDefault() }'
```

## Additional Usage

### Custom Events

**jquery-attach** can handle Custom Events by using jQuery's `trigger` function.

See example below:

```
var $invert = $('.invert');
var $font = $('.font');
var $content = $('.content');

// Attach Custom Events
$content
  .attachBehavior('invertColor', function (e) {
    $content.toggleClass('invert');
  })
  .attachBehavior('toggleFont', function (e) {
    $content.toggleClass('font');
  });

// Trigger "invertColor" Custom Event
$invert.attachBehavior('click', function (e) {
  $content.trigger('invertColor');
});

// Trigger "toggleFont" Custom Event
$font.attachBehavior('click', function (e) {
  $content.trigger('toggleFont');
});
```

### Error Handling

There are two errors that can **jquery-attach** might throw

- When multiple behaviors have been added to an element's event (e.g. multiple `click` events for a single button)
- When the callback argument was passed something that is not a function (e.g. `attachBehavior('click', true)`)

Errors can be handled by using the `try...catch` statment:

```
try {
  $('.link').attachBehavior('click', function (e) {
    e.preventDefault();
  });
}
catch (err) {
  console.error(err);
  // handle error here
}
```

## Other Notes

- **jquery-attach** supports behavior assignment for elements in the same group or selector (e.g. `$('.nav-links')`)
- Using jQuery's `on` function will also prevent assignment of new behaviors on the same event. (See example below)

```
$('.link').on('click', function(){});
$('.link').attachBehavior('click', function(){});

// the second line above will throw an error
// saying that the 'click' event
// already has a behavior attached to it
```

---

## Changelog:

**1.2.1**:

- Fixed `detachBehavior` to just detach the behavior for the specified event

**1.2.0**:

- Added Custom Events binding
- Added detection of bindings using jQuery "on" function
- Added examples [`/examples`](/examples)
- Added tests [`/tests`](/tests)
- Added chaining
- Added `showErrors` option
- Updated source code to ES6 (transpiled to ES5)
- Updated error handling

**1.1.2**:

- Fixed references to the code repository


**1.1.1**:

- Published plugin to bower
- Updated license from MIT to ISC

**1.1.0**:

- Enclosed the plugin inside an IIFE
- Applied mangling on `jquery-attach.min.js`

**1.0.1**:

- Fixed the `$ is undefined error` by using universal `jQuery` method

---

## To-Do
- [x] Add custom events binding
- [x] Add examples
- [x] Add test cases
