/*
 * jQuery Attach
 * Simple jQuery plugin to ensure one-time behavior binding
 * (c) 2016 Joshua Cerbito - https://github.com/joshuacerbito/jquery-attach-behavior.git
 * Released under the ISC license
 */

(function($) {
  'use strict';

  const typeOf = function (str) {
    return Object.prototype.toString.call(str).replace(/^\[object\s(.*)\]$/, "$1")
  };

  $.fn.attachBehavior = function(trigger = '', cb = function(){}, showErrors = true) {
    let _showErrors = !!showErrors;
    let _this = this;
    let _jqueryEvent = $._data($(_this)[0], 'events') || {};

    if ( !_this.hasOwnProperty('binding') ) {
      _this.binding = {};
    }

    if ( _showErrors && _jqueryEvent.hasOwnProperty(trigger) ) {
      throw Error(`The "${trigger}" event has already been assigned (using jQuery's 'on' function) to this behavior:\n${_jqueryEvent[trigger][0].handler}`);
    }

    if ( _showErrors && _this.binding.hasOwnProperty(trigger) ) {
      throw Error(`The "${trigger}" event has already been assigned to this behavior:\n${cb}`);
    }

    if ( _showErrors && (typeof cb !== 'function') ) {
      throw TypeError(`Expected a "function", instead saw a "${typeOf(cb)}": "${cb}"`);
    }

    _this.on(trigger, function(evt) {
      cb.apply(_this, arguments);
    });

    _this.binding[trigger] = cb;

    return _this;
  };

  $.fn.detachBehavior = function(trigger) {
    let _this = this;
    let _jqueryEvent = $._data($(_this)[0], 'events') || {};

    if ( !!_this.hasOwnProperty('binding') ) {
      if( !!_this.binding.hasOwnProperty(trigger) ) {
        _this.off(trigger);
      }
    }

    _this.binding[trigger] = null;
    delete _this.binding[trigger];

    return _this;
  };

  $.fn.getBehavior = function() {
    return this.binding;
  };
})(jQuery);
